<?php

namespace InternetGalerie\Igshop2\Utility;

use TYPO3\CMS\Core\SingletonInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Extbase\Service\ImageService;
use TYPO3\CMS\Core\Resource\FileReference;
use TYPO3\CMS\Core\Resource\ResourceFactory;
use TYPO3\CMS\Core\Resource\Exception\ResourceDoesNotExistException;

class AjaxUtility implements SingletonInterface
{
    public function __construct(
        private readonly ResourceFactory $resourceFactory,
        private readonly ImageService $imageService,
    ) {}

    public function getImageIds($productUid, $fieldName, $tablename)
    {
        // Fetch image IDs from the database using the product UID and field name
        $connection = GeneralUtility::makeInstance(ConnectionPool::class)
            ->getConnectionForTable('sys_file_reference');

        $queryBuilder = $connection->createQueryBuilder();
        $rows = $queryBuilder
            ->select('uid_local')
            ->from('sys_file_reference')
            ->where(
                $queryBuilder->expr()->eq('uid_foreign', $queryBuilder->createNamedParameter($productUid, \PDO::PARAM_INT)),
                $queryBuilder->expr()->eq('tablenames', $queryBuilder->createNamedParameter($tablename, \PDO::PARAM_STR)),
                $queryBuilder->expr()->eq('fieldname', $queryBuilder->createNamedParameter($fieldName, \PDO::PARAM_STR))
            )
            ->execute()
            ->fetchAll();

        // Extract image IDs
        $imageIds = array_column($rows, 'uid_local');
        return $imageIds;
    }
    
    public function getImagePaths(array $imageIds, ?int $width = null, ?int $height = null)
    {
        $imagePaths = [];
        $processingInstructions = [];
        if ($width > 0) {
            $processingInstructions['width'] = $width . 'c'; // c = crop if necessary
        }
        if ($height > 0) {
            $processingInstructions['height'] = $height . 'c'; // c = crop if necessary
        }

        
        foreach ($imageIds as $imageId) {
            $file = $this->resourceFactory->getFileObject($imageId);
            if ($file) {
                if ($width > 0) {
                    $processedImage = $this->imageService->applyProcessingInstructions($file, $processingInstructions);
                    $filePath = $this->imageService->getImageUri($processedImage);
                } else {
                    $filePath = $file->getPublicUrl();
                }
                $imagePaths[] = $filePath;
            }
        }

        return $imagePaths;
    }
    /*
    public function getImagePaths($imageIds)
    {
        $imagePaths = [];
        
        foreach ($imageIds as $imageId) {
            try {
                $fileReference = $this->resourceFactory->getFileReferenceObject($imageId);
                if ($fileReference) {
                    $filePath = $fileReference->getPublicUrl();
                    $imagePaths[] = $filePath;
                }
            } catch (ResourceDoesNotExistException $e) {
                $imagePaths[] = $imageId;
            }
        }

        return $imagePaths;
    }
    */
}