<?php

namespace InternetGalerie\Igshop2\Utility;

use Ig\IgPayment\Services\PaymentService;
use Ig\IgPayment\Utility\PaymentUtility;
use TYPO3\CMS\Core\SingletonInterface;
use InternetGalerie\Igshop2\Domain\Model\Order;
use InternetGalerie\Igshop2\Domain\Model\Country;
use InternetGalerie\Igshop2\Domain\Repository\CountryRepository;
use InternetGalerie\Igshop2\Domain\Repository\OrderProductRepository;
use InternetGalerie\Igshop2\Domain\Repository\OrderRepository;
use InternetGalerie\Igshop2\Domain\Repository\PortoRepository;
use InternetGalerie\Igshop2\Domain\Repository\ProductRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;

class CartUtility implements SingletonInterface
{
    /**
     * the current order
     *
     * @var Order
     */
    public $order = null;

    /**
     * @var string Name of the extension this view helper belongs to
     */
    protected $extensionName = 'IgShop2';

    /**
     * @var string Name of the extension this view helper belongs to (ext_localconf.php configurePlugin zweites  Argument)
     */
    protected $pluginName = 'igshop2';

    /**
     * orderRepository
     *
     * @var OrderRepository
     */
    protected $orderRepository = null;

    /**
     * productRepository
     *
     * @var ProductRepository
     **/
    protected $productRepository = null;

    /**
     * OrderProductRepository
     *
     * @var OrderProductRepository
     */
    protected $orderProductRepository = null;

    /**
     * countryRepository
     *
     * @var CountryRepository
     */
    protected $countryRepository = null;

    /**
     * portoRepository
     *
     * @var PortoRepository
     */
    protected $portoRepository = null;


    /**
     * @var ConfigurationManagerInterface
     */
    protected $configurationManager;


    protected PaymentUtility $paymentUtility;
    
    /**
     * country used just for default ? [DA] @todo remove
     *
     * @var Country
     */
    protected $country = null;

    protected $settings = null;


    public function injectConfigurationManager(ConfigurationManagerInterface $configurationManager): void
    {
        $this->configurationManager = $configurationManager;
        $this->settings = $this->configurationManager->getConfiguration(
            ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS,
            $this->extensionName,
            $this->pluginName
        );
    }

    public function injectPaymentUtility(PaymentUtility $paymentUtility): void
    {
        $this->paymentUtility = $paymentUtility;
    }

    public function injectOrderRepository(OrderRepository $orderRepository): void
    {
        $this->orderRepository = $orderRepository;
    }

    public function injectProductRepository(ProductRepository $productRepository): void
    {
        $this->productRepository = $productRepository;
    }

    public function injectOrderProductRepository(OrderProductRepository $orderProductRepository): void
    {
        $this->orderProductRepository = $orderProductRepository;
    }


    public function injectCountryRepository(CountryRepository $countryRepository): void
    {
        $this->countryRepository = $countryRepository;
    }


    public function injectPortoRepository(PortoRepository $portoRepository): void
    {
        $this->portoRepository = $portoRepository;
    }

    /**
     * Aktueller Warenkorb laden
     */
    public function initializeObject(): void
    {
        $this->country = $this->countryRepository->getCountryFromRemoteAddr($_SERVER['REMOTE_ADDR']);
        $currency = $this->country->getCurrency();
        $this->order = $this->orderRepository->getCurrentOrder($currency);
        $this->setCurrentOrderToPayment();
    }

    // OrderController.php ist doch in order drin??????
    public function getCountry()
    {
        return $this->country;
    }

    public function getCartByUid($uid): void
    {
        $this->order = $this->orderRepository->findByUid($uid);
        $this->setCurrentOrderToPayment();
    }


    // von ProductController.php fuer Waehrung ???
    /*
    public function getCurrency(Currency $currency)
    {
        $this->getCart( $currency );
        return $this->order->getCurrency();
    }
    */

    public function addProduct($product, $options = []): void
    {
        $this->order->addNewProduct($product, $this->settings['prodLimit'] ?? null, $options);
    }

    public function addProductByUid(int $productUid, $options = []): void
    {
        $product = $this->productRepository->findByUid($productUid);
        $this->addProduct($product, $options);
    }

    public function removeOrderProduct($orderProduct): void
    {
        $this->order->removeProduct($orderProduct);
    }

    public function removeOrderProductByUid(int $orderProductUid, $options = []): void
    {
        $orderProduct = $this->orderProductRepository->findByUid($orderProductUid);
        if (is_object($orderProduct)) {
            $orderProduct->setAmount(0);
            $this->orderProductRepository->update($orderProduct);
            $this->removeOrderProduct($orderProduct);
        }
    }

    public function orderProductSetQuantity($orderProduct, $quantity)
    {
        return $this->order->changeProductQuantity($orderProduct, $quantity);
    }

    public function orderProductSetQuantityByUid(int $orderProductUid, $quantity)
    {
        $orderProduct = $this->orderProductRepository->findByUid($orderProductUid);
        if (is_object($orderProduct)) {
            return $this->orderProductSetQuantity($orderProduct, $quantity);
        }

        return false;
    }

    // Bestellung ist mit Porto?
    public function hasPorto()
    {
        $paymentMethod = $this->order->getPaymentMethod();
        // Ist Bestellung ohne Porto
        if (!$paymentMethod || $paymentMethod->getNoPorto() || $this->order->hasProductWithoutPorto()) {
            return false;
        }

        return true;
    }

    public function setPorto($portoUid): void
    {
        if ($this->hasPorto()) {
            $country = $this->order->getCountry();
            if ($portoUid > 0) {
                $porto = $this->portoRepository->findByUid($portoUid);
                if (is_object($porto) && $this->order->isPortoValidWithCountry($country)) {
                    $this->order->setPorto($porto);
                    $this->update();
                    return;
                }

                // no valid porto -> Fehler ?!
                //$this->order->removePorto();
            }

            $this->setDefaultPorto($country);
        } else {
            $this->order->removePorto();
        }
    }

    // Default Porto setzen falls leer -> wirklich noetig? [DA]
    public function setDefaultPorto($country = null): void
    {
        if ($country === null) {
            // @todo Mist - warum Porto mit Uid=1 [DA]  fuer OrderController showAction
            $porto = $this->portoRepository->findByUid(1);
        } else {
            // @todo neu $this->shop->getDefaultPortoForCountry($country)
            $porto = $this->portoRepository->getDefaultPortoForCountry($country);
        }

        if (is_object($porto)) {
            $this->order->setPorto($porto);
            $this->update();
        } else {
            // ist eh leer oder ungueltig?
            $this->order->removePorto();
        }
    }

    public function getAvailablePortos()
    {
        if ($this->hasPorto()) {
            $country = $this->order->getCountry();
            // @todo neu $this->shop->getPortosForCountry($country)
            return $this->portoRepository->getAllForCountry($country);
        }

        return [];
    }

    public function setSubmitted($submitted): void
    {
        $this->order->setSubmitted($submitted);
    }
    public function setCurrentOrderToPayment(): ?PaymentService
    {
        //var_dump($this->order);die();
        return $this->paymentUtility->setCurrentService(
            $this->order->getPaymentMethod()?->getCode(), //@todo php 7.4
            $this->order->getCurrency()?->getShort(),
            $this->order->getTotalWithPorto(),
            $this->order->getUid() . '-' . $this->order->getCheckoutReference(),
        );
    }

    public function update($updatePayment = true): void
    {
        if($updatePayment) {
            $this->setCurrentOrderToPayment();
            //$this->order->updatePayment();
        }
        $this->orderRepository->persistOrder($this->order);
    }
}
